const { readdirSync } = require('fs')
const path = require('path')
const getNodeModulesPath = require('../getNodeModulesPath')
const { Logger } = require('../Logger');
const { getRepoName } = require('../../pre-commit/gitSetup/gitUtils/gitActions')
const { getLibraryInstalledLocation } = require('../getLibraryInstalledLocation')
const { executeSynchronizedCommands } = require('../executeSyncCommands')
const {repoURL} = require('../../preCommitRequestUtils.json')
const { handleExceptionCase } = require('../getScripts')
const nodeModulesPathOfProject = getNodeModulesPath()

let packageJsonFilePath = path.join(nodeModulesPathOfProject,'package.json')

function getUnInstalledPlugins(pluginsForCurrentRepo,pluginNamesOfDevDependencyPlugins,devDependencies){
    let pluginsInNodeModules = executeSynchronizedCommands(readdirSync,[path.join(nodeModulesPathOfProject,'node_modules')],'','Unable to get the plugins inside node_modules',true,false)
    let pluginsToBeInstalled = []
    pluginsForCurrentRepo.filter(plugin => {
        if(plugin.packageName.startsWith('@')){
            let scope = plugin.packageName.split('/')[0]
            let pluginName = plugin.packageName.split('/')[1]
            let scopedPlugins = executeSynchronizedCommands(readdirSync,[path.join(nodeModulesPathOfProject,'node_modules',scope)],'','Unable to get the plugins inside the scope inside node_modules',true,false)
            let isPluginInstalled = scopedPlugins.includes(pluginName) ? true : false
            if(!isPluginInstalled){
                pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`)
            }
        }
        else{
            let isPluginInstalled = pluginsInNodeModules.includes(plugin.packageName) ? true : false
            if(!isPluginInstalled){
                pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`)
            }
        }
        if(pluginNamesOfDevDependencyPlugins.includes(plugin.packageName)){
            if(plugin.version !== devDependencies[plugin.packageName]){
                pluginsToBeInstalled.push(`${plugin.packageName}@${plugin.version}`)
            }
        }
    })
    return pluginsToBeInstalled
}

function getServiceSpecificPluginsPath(){
    const repoName = getRepoName()
    const pathToPLuginsFile = path.join(getLibraryInstalledLocation(),'configuration','services',repoName,'pluginVersion.js')
    return repoName ? pathToPLuginsFile : undefined
}
function getAllPlugins(){
    let serviceSpecificPlugins = []
    const pathToCommonPluginsFile = path.join(getLibraryInstalledLocation(),'configuration','common','pluginVersion.js')
    let commonPlugins = handleExceptionCase([],'Make sure there is a pluginVersion.js file in common folder under configuration folder',require,pathToCommonPluginsFile)

    const serviceSpecificPluginsFile = getServiceSpecificPluginsPath()
    if(serviceSpecificPluginsFile !== undefined){
        serviceSpecificPlugins = handleExceptionCase([],`Kindly Ensure that Configuration is setup in Configuration repo \n ${repoURL}`,require,serviceSpecificPluginsFile);
    }
    else{
        serviceSpecificPlugins = []
    }
    return [...commonPlugins,...serviceSpecificPlugins]

}
function checkIfPluginsAreInstalled(){
    const pluginsForCurrentRepo = getAllPlugins()
    let uninstalledPlugins = []
    let plugin = {
        noPluginMessage: '',
        uninstalledPlugins: []
    }
    let packageJsonContent = require(packageJsonFilePath)
    let pluginsIndevDependencies = packageJsonContent.devDependencies
    let pluginNamesOfDevDependencyPlugins = []
    let arePluginsPresentIndevDependencies = false
    let arePluginsPresentForCurrentRepo = false

    if(pluginsIndevDependencies !== undefined){
        arePluginsPresentIndevDependencies = true
        pluginNamesOfDevDependencyPlugins = Object.keys(pluginsIndevDependencies)
    }
    else if(pluginsIndevDependencies === undefined){
        Logger.log(Logger.INFO_TYPE,'No devDependencies present!')
    }
    arePluginsPresentForCurrentRepo = pluginsForCurrentRepo.length !== 0 ? true : false

    if(arePluginsPresentForCurrentRepo || arePluginsPresentIndevDependencies){
        uninstalledPlugins = getUnInstalledPlugins(pluginsForCurrentRepo,pluginNamesOfDevDependencyPlugins,pluginsIndevDependencies)
        plugin.noPluginMessage = ''
        plugin.uninstalledPlugins = uninstalledPlugins
        return plugin
        
    }
    else{
        plugin.noPluginMessage = 'No plugins present for the repository'
        plugin.uninstalledPlugins = []
        return plugin
    }   
}

module.exports = {
    checkIfPluginsAreInstalled
}
